@@ -1,4 +1,3 @@
-import itertools
 import logging
 
 import engineio
@@ -14,12 +13,10 @@ class Server(object):
     This class implements a fully compliant Socket.IO web server with support
     for websocket and long-polling transports.
 
-    :param engineio_options: A ``dict`` with options for the Engine.IO server.
-                            The values are passed directly to the
-                            ``engineio.Server`` constructor.
-    :param client_manager_class: The class that will manage the client list.
-                                 The default value is appropriate for most
-                                 cases.
+    :param client_manager: The client manager instance that will manage the
+                           client list. By default the client list is stored
+                           in an in-memory structure, which prevents the use
+                           of multiple worker processes.
     :param logger: To enable logging set to ``True`` or pass a logger object to
                    use. To disable logging set to ``False``.
     :param binary: ``True`` to support binary payloads, ``False`` to treat all
@@ -28,6 +25,10 @@ class Server(object):
                    ``bytes`` values are treated as binary.  This option has no
                    effect on Python 3, where text and binary payloads are
                    always automatically discovered.
+    :param json: An alternative json module to use for encoding and decoding
+                 packets. Custom json modules must have ``dumps`` and ``loads``
+                 functions that are compatible with the standard library
+                 versions.
     :param kwargs: Connection parameters for the underlying Engine.IO server.
 
     The Engine.IO configuration supports the following settings:
@@ -59,15 +60,18 @@ class Server(object):
                             a logger object to use. To disable logging set to
                             ``False``.
     """
-    def __init__(self, client_manager_class=None, logger=False, binary=False,
-                 **kwargs):
-        if client_manager_class is None:
-            client_manager_class = base_manager.BaseManager
-        self.manager = client_manager_class(self)
+    def __init__(self, client_manager=None, logger=False, binary=False,
+                 json=None, **kwargs):
+        if client_manager is None:
+            client_manager = base_manager.BaseManager(self)
+        self.manager = client_manager
         engineio_options = kwargs
         engineio_logger = engineio_options.pop('engineio_logger', None)
         if engineio_logger is not None:
             engineio_options['logger'] = engineio_logger
+        if json is not None:
+            packet.Packet.json = json
+            engineio_options['json'] = json
         self.eio = engineio.Server(**engineio_options)
         self.eio.on('connect', self._handle_eio_connect)
         self.eio.on('message', self._handle_eio_message)
@@ -76,7 +80,6 @@ class Server(object):
 
         self.environ = {}
         self.handlers = {}
-        self.callbacks = {}
 
         self._binary_packet = None
         self._attachment_count = 0
@@ -143,7 +146,7 @@ class Server(object):
             return set_handler
         set_handler(handler)
 
-    def emit(self, event, data, room=None, skip_sid=None, namespace=None,
+    def emit(self, event, data=None, room=None, skip_sid=None, namespace=None,
              callback=None):
         """Emit a custom event to one or more connected clients.
 
@@ -271,6 +274,16 @@ class Server(object):
         self._trigger_event('disconnect', namespace, sid)
         self.manager.disconnect(sid, namespace=namespace)
 
+    def transport(self, sid):
+        """Return the name of the transport used by the client.
+
+        The two possible values returned by this function are ``'polling'``
+        and ``'websocket'``.
+
+        :param sid: The session of the client.
+        """
+        return self.eio.transport(sid)
+
     def handle_request(self, environ, start_response):
         """Handle an HTTP request from the client.
 
@@ -287,12 +300,8 @@ class Server(object):
         """
         return self.eio.handle_request(environ, start_response)
 
-    def _emit_internal(self, sid, event, data, namespace=None, callback=None):
+    def _emit_internal(self, sid, event, data, namespace=None, id=None):
         """Send a message to a client."""
-        if callback is not None:
-            id = self._generate_ack_id(sid, namespace, callback)
-        else:
-            id = None
         if six.PY2 and not self.binary:
             binary = False  # pragma: nocover
         else:
@@ -336,13 +345,9 @@ class Server(object):
             if n != '/' and self.manager.is_connected(sid, n):
                 self._trigger_event('disconnect', n, sid)
                 self.manager.disconnect(sid, n)
-                if sid in self.callbacks and n in self.callbacks[sid]:
-                    del self.callbacks[sid][n]
         if namespace == '/' and self.manager.is_connected(sid, namespace):
             self._trigger_event('disconnect', '/', sid)
             self.manager.disconnect(sid, '/')
-            if sid in self.callbacks:
-                del self.callbacks[sid]
             if sid in self.environ:
                 del self.environ[sid]
 
@@ -373,34 +378,13 @@ class Server(object):
         """Handle ACK packets from the client."""
         namespace = namespace or '/'
         self.logger.info('received ack from %s [%s]', sid, namespace)
-        self._trigger_callback(sid, namespace, id, data)
+        self.manager.trigger_callback(sid, namespace, id, data)
 
     def _trigger_event(self, event, namespace, *args):
         """Invoke an application event handler."""
         if namespace in self.handlers and event in self.handlers[namespace]:
             return self.handlers[namespace][event](*args)
 
-    def _generate_ack_id(self, sid, namespace, callback):
-        """Generate a unique identifier for an ACK packet."""
-        namespace = namespace or '/'
-        if sid not in self.callbacks:
-            self.callbacks[sid] = {}
-        if namespace not in self.callbacks[sid]:
-            self.callbacks[sid][namespace] = {0: itertools.count(1)}
-        id = six.next(self.callbacks[sid][namespace][0])
-        self.callbacks[sid][namespace][id] = callback
-        return id
-
-    def _trigger_callback(self, sid, namespace, id, data):
-        """Invoke an application callback."""
-        namespace = namespace or '/'
-        try:
-            callback = self.callbacks[sid][namespace][id]
-        except KeyError:
-            raise ValueError('Unknown callback')
-        del self.callbacks[sid][namespace][id]
-        callback(*data)
-
     def _handle_eio_connect(self, sid, environ):
         """Handle the Engine.IO connection event."""
         self.environ[sid] = environ
